@@ -22,20 +22,22 @@
 """
 Main application
 """
-import os
 import gobject
 import gtk
 import gst
-import sys
 import check
 from ui import mainwindow
-from discoverer import Discoverer
+from pitivigstutils import patch_gst_python
 from playground import PlayGround
 from project import Project, file_is_project
 from effects import Magician
 from configure import APPNAME
+from settings import GlobalSettings
+from threads import ThreadMaster
 import instance
 
+from gettext import gettext as _
+
 class Pitivi(gobject.GObject):
     """
     Pitivi's main class
@@ -45,6 +47,7 @@ class Pitivi(gobject.GObject):
       closing-project : Pitivi wishes to close the project, callbacks return False
                 if they don't want the project to be closed, True otherwise
       not-project : The given uri is not a project file
+      shutdown : PiTiVi has shutdown
 
     """
     __gsignals__ = {
@@ -56,32 +59,45 @@ class Pitivi(gobject.GObject):
                               (gobject.TYPE_PYOBJECT, )),
         "not-project" : ( gobject.SIGNAL_RUN_LAST,
                           gobject.TYPE_NONE,
-                          (gobject.TYPE_STRING, ))
+                          (gobject.TYPE_STRING, )),
+        "shutdown" : ( gobject.SIGNAL_RUN_LAST,
+                       gobject.TYPE_NONE,
+                       ( ))
         }
 
     project = None
 
-    def __init__(self, *args):
+    def __init__(self, use_ui=True, *args, **kwargs):
         """
         initialize pitivi with the command line arguments
         """
         gst.log("starting up pitivi...")
         gobject.GObject.__init__(self)
 
+        self._use_ui = use_ui
+
+        # patch gst-python for new behaviours
+        patch_gst_python()
+
         # store ourself in the instance global
         if instance.PiTiVi:
-            raise RuntimeWarning("There is already a %s instance, inform developers" % APPNAME)
+            raise RuntimeWarning(_("There is already a %s instance, inform developers") % APPNAME)
         instance.PiTiVi = self
-        
+
         # TODO parse cmd line arguments
 
+        # get settings
+        self.settings = GlobalSettings()
+        self.threads = ThreadMaster()
+
         self.playground = PlayGround()
-        self.current = Project("New Project")
+        self.current = Project(_("New Project"))
         self.effects = Magician()
-        
-        # we're starting a GUI for the time being
-        self.gui = mainwindow.PitiviMainWindow()
-        self.gui.show()
+
+        if self._use_ui:
+            # we're starting a GUI for the time being
+            self.gui = mainwindow.PitiviMainWindow()
+            self.gui.show()
 
     def loadProject(self, uri=None, filepath=None):
         """ Load the given file through it's uri or filepath """
@@ -109,26 +125,34 @@ class Pitivi(gobject.GObject):
             self.playground.pause()
             self.current = None
         return True
-        
+
     def newBlankProject(self):
         """ start up a new blank project """
         # if there's a running project we must close it
         if self._closeRunningProject():
             self.playground.pause()
-            self.current = Project("New Project")
+            self.current = Project(_("New Project"))
             self.emit("new-project", self.current)
 
     def shutdown(self):
         """ close PiTiVi """
         gst.debug("shutting down")
-        if not self._closeRunningProject():
+        # we refuse to close if we're running a user interface and the user
+        # doesn't want us to close the current project.
+        if self._use_ui and not self._closeRunningProject():
+            gst.warning("Not closing since running project doesn't want to close")
             return
+        self.threads.stopAllThreads()
         self.playground.shutdown()
-        gst.debug("Exiting main loop")
-        gtk.main_quit()
+        instance.PiTiVi = None
+        self.emit("shutdown")
+
+def shutdownCb(pitivi):
+    gst.debug("Exiting main loop")
+    gtk.main_quit()
 
-        
 def main(argv):
     check.initial_checks()
     ptv = Pitivi(argv)
+    ptv.connect('shutdown', shutdownCb)
     gtk.main()
