@@ -31,13 +31,9 @@ from glade import GladeWindow
 import plumber
 import pitivi.instance as instance
 from pitivi.bin import SmartTimelineBin
-from pitivi.objectfactory import FileSourceFactory
 import pitivi.dnd as dnd
-from pitivi.settings import ExportSettings
 from pitivi.signalgroup import SignalGroup
 
-from exportsettingswidget import ExportSettingsDialog
-
 from gettext import gettext as _
 
 def time_to_string(value):
@@ -79,10 +75,10 @@ class PitiviViewer(gtk.VBox):
 
     def _connectToProject(self, project):
         """Connect signal handlers to a project.
- 
+
         This first disconnects any handlers connected to an old project.
         If project is None, this just disconnects any connected handlers.
- 
+
         """
         self.project_signals.connect(project.sources, "tmp_is_ready",
                                      None, self._tmpIsReadyCb)
@@ -93,16 +89,13 @@ class PitiviViewer(gtk.VBox):
         """ Creates the Viewer GUI """
         self.set_border_width(5)
         self.set_spacing(5)
-        
+
         # drawing area
         self.aframe = gtk.AspectFrame(xalign=0.5, yalign=0.5, ratio=4.0/3.0, obey_child=False)
         self.pack_start(self.aframe, expand=True)
         self.drawingarea = ViewerWidget()
         self.drawingarea.connect_after("realize", self._drawingAreaRealizeCb)
         self.aframe.add(self.drawingarea)
-        
-        # horizontal line
-        self.pack_start(gtk.HSeparator(), expand=False)
 
         # Slider
         self.posadjust = gtk.Adjustment()
@@ -114,23 +107,18 @@ class PitiviViewer(gtk.VBox):
         self.pack_start(self.slider, expand=False)
         self.moving_slider = False
         self.slider.set_sensitive(False)
-        
+
         # Buttons/Controls
         bbox = gtk.HBox()
         boxalign = gtk.Alignment(xalign=0.5, yalign=0.5)
         boxalign.add(bbox)
         self.pack_start(boxalign, expand=False)
 
-        self.record_button = gtk.ToolButton(gtk.STOCK_MEDIA_RECORD)
-        self.record_button.connect("clicked", self._recordCb)
-        self.record_button.set_sensitive(False)
-        bbox.pack_start(self.record_button, expand=False)
-        
         self.rewind_button = gtk.ToolButton(gtk.STOCK_MEDIA_REWIND)
         self.rewind_button.connect("clicked", self._rewindCb)
         self.rewind_button.set_sensitive(False)
         bbox.pack_start(self.rewind_button, expand=False)
-        
+
         self.back_button = gtk.ToolButton(gtk.STOCK_MEDIA_PREVIOUS)
         self.back_button.connect("clicked", self._backCb)
         self.back_button.set_sensitive(False)
@@ -140,12 +128,12 @@ class PitiviViewer(gtk.VBox):
         self.playpause_button.connect("play", self._playButtonCb)
         bbox.pack_start(self.playpause_button, expand=False)
         self.playpause_button.set_sensitive(False)
-        
+
         self.next_button = gtk.ToolButton(gtk.STOCK_MEDIA_NEXT)
         self.next_button.connect("clicked", self._nextCb)
         self.next_button.set_sensitive(False)
         bbox.pack_start(self.next_button, expand=False)
-        
+
         self.forward_button = gtk.ToolButton(gtk.STOCK_MEDIA_FORWARD)
         self.forward_button.connect("clicked", self._forwardCb)
         self.forward_button.set_sensitive(False)
@@ -155,9 +143,9 @@ class PitiviViewer(gtk.VBox):
         self.timelabel = gtk.Label()
         self.timelabel.set_markup("<tt>00m00s000 / --m--s---</tt>")
         self.timelabel.set_alignment(1.0, 0.5)
-        self.timelabel.set_padding(5, 5)        
+        self.timelabel.set_padding(5, 5)
         bbox.pack_start(self.timelabel, expand=False, padding=10)
-        
+
 
         # drag and drop
         self.drag_dest_set(gtk.DEST_DEFAULT_DROP | gtk.DEST_DEFAULT_MOTION,
@@ -168,7 +156,7 @@ class PitiviViewer(gtk.VBox):
     def _asyncFrameRatioChange(self, ratio):
         gst.debug("ratio:%f" % ratio)
         self.aframe.set_property("ratio", ratio)
-        
+
     def _videosinkCapsNotifyCb(self, sinkpad, unused_property):
         caps = sinkpad.get_negotiated_caps()
         if not caps:
@@ -315,12 +303,9 @@ class PitiviViewer(gtk.VBox):
 
     def _asyncTimelineDurationChanged(self, duration):
         gst.debug("duration : %d" % duration)
-        # deactivate record button is the duration is null
-        self.record_button.set_sensitive((duration > 0) and True or False)
-            
         self.posadjust.upper = float(duration)
         self.timelabel.set_markup("<tt>%s / %s</tt>" % (time_to_string(self.current_time), time_to_string(instance.PiTiVi.playground.current.length)))
-        
+
 
     def _backToDefaultCb(self):
         instance.PiTiVi.playground.switchToDefault()
@@ -361,27 +346,18 @@ class PitiviViewer(gtk.VBox):
         """ the current project has changed """
         assert(instance.PiTiVi.current == project)
         self._connectToProject(project)
-        
+
     def _addTimelineToPlayground(self):
         instance.PiTiVi.playground.addPipeline(instance.PiTiVi.current.getBin())
 
 
     ## Control gtk.Button callbacks
 
-    def _encodingDialogDestroyCb(self, unused_dialog):
-        instance.PiTiVi.gui.set_sensitive(True)
-        
-    def _recordCb(self, unused_button):
-        win = EncodingDialog(instance.PiTiVi.current)
-        win.window.connect("destroy", self._encodingDialogDestroyCb)
-        instance.PiTiVi.gui.set_sensitive(False)
-        win.show()
-
     def _rewindCb(self, unused_button):
         pass
 
     def _backCb(self, unused_button):
-        pass
+        instance.PiTiVi.playground.backward_one()
 
     def _playButtonCb(self, unused_button, isplaying):
         if isplaying:
@@ -392,14 +368,14 @@ class PitiviViewer(gtk.VBox):
                 self.currentState = gst.STATE_PAUSED
 
     def _nextCb(self, unused_button):
-        pass
+        instance.PiTiVi.playground.forward_one()
 
     def _forwardCb(self, unused_button):
         pass
 
 
     ## Playground callbacks
-    
+
     def _playgroundPositionCb(self, unused_playground, unused_smartbin, pos):
         self._newTime(pos)
 
@@ -408,23 +384,21 @@ class PitiviViewer(gtk.VBox):
         if smartbin == playground.default:
             self.slider.set_sensitive(False)
             self.playpause_button.set_sensitive(False)
-            self.record_button.set_sensitive(False)
+            self.next_button.set_sensitive(False)
+            self.back_button.set_sensitive(False)
         else:
             if isinstance(smartbin, SmartTimelineBin):
                 gst.info("switching to Timeline, setting duration to %s" % (gst.TIME_ARGS(smartbin.project.timeline.videocomp.duration)))
                 self.posadjust.upper = float(smartbin.project.timeline.videocomp.duration)
                 smartbin.project.timeline.videocomp.connect("start-duration-changed",
                                                             self._timelineDurationChangedCb)
-                if smartbin.project.timeline.videocomp.duration > 0:
-                    self.record_button.set_sensitive(True)
-                else:
-                    self.record_button.set_sensitive(False)
             else:
                 self.posadjust.upper = float(smartbin.factory.length)
-                self.record_button.set_sensitive(False)
             self._newTime(0)
             self.slider.set_sensitive(True)
             self.playpause_button.set_sensitive(True)
+            self.next_button.set_sensitive(True)
+            self.back_button.set_sensitive(True)
 
     def _currentStateCb(self, unused_playground, state):
         gst.info("current state changed : %s" % state)
@@ -495,7 +469,7 @@ class PlayPauseButton(gtk.Button):
         gtk.Button.__init__(self, label="")
         self.playing = True
         self.setPlay()
-        self.connect('clicked', self._clickedCb)        
+        self.connect('clicked', self._clickedCb)
 
     def _clickedCb(self, unused):
         if not self.playing:
@@ -519,96 +493,4 @@ class PlayPauseButton(gtk.Button):
             self.playing = True
 
 
-class EncodingDialog(GladeWindow):
-    """ Encoding dialog box """
-    glade_file = "encodingdialog.glade"
-
-    def __init__(self, project):
-        GladeWindow.__init__(self)
-        self.project = project
-        self.bin = project.getBin()
-        self.bus = self.bin.get_bus()
-        self.bus.add_signal_watch()
-        self.eosid = self.bus.connect("message::eos", self._eosCb)
-        self.outfile = None
-        self.progressbar = self.widgets["progressbar"]
-        self.cancelbutton = self.widgets["cancelbutton"]
-        self.positionhandler = 0
-        self.rendering = False
-        self.settings = project.getSettings()
-        self.timestarted = 0
-        self.vinfo = self.widgets["videoinfolabel"]
-        self.ainfo = self.widgets["audioinfolabel"]
-        self._displaySettings()
-
-    def _displaySettings(self):
-        self.vinfo.set_markup(self.settings.getVideoDescription())
-        self.ainfo.set_markup(self.settings.getAudioDescription())
-
-    def _fileButtonClickedCb(self, button):
-        
-        dialog = gtk.FileChooserDialog(title=_("Choose file to render to"),
-                                       parent=self.window,
-                                       buttons=(gtk.STOCK_OK, gtk.RESPONSE_ACCEPT,
-                                                gtk.STOCK_CANCEL, gtk.RESPONSE_REJECT),
-                                       action=gtk.FILE_CHOOSER_ACTION_SAVE)
-        if self.outfile:
-            dialog.set_current_name(self.outfile)
-        res = dialog.run()
-        dialog.hide()
-        if res == gtk.RESPONSE_ACCEPT:
-            self.outfile = dialog.get_uri()
-            button.set_label(os.path.basename(self.outfile))
-        dialog.destroy()
-
-    def _positionCb(self, unused_playground, unused_smartbin, position):
-        timediff = time.time() - self.timestarted
-        self.progressbar.set_fraction(float(position) / float(self.bin.length))
-        if timediff > 5.0:
-            # only display ETA after 5s in order to have enough averaging
-            totaltime = (timediff * float(self.bin.length) / float(position)) - timediff
-            self.progressbar.set_text(_("Finished in %dm%ds") % (int(totaltime) / 60,
-                                                              int(totaltime) % 60))
-
-    def _recordButtonClickedCb(self, unused_button):
-        if self.outfile and not self.rendering:
-            if self.bin.record(self.outfile, self.settings):
-                self.timestarted = time.time()
-                self.positionhandler = instance.PiTiVi.playground.connect('position', self._positionCb)
-                self.rendering = True
-                self.cancelbutton.set_label("gtk-cancel")
-                self.progressbar.set_text(_("Rendering"))
-            else:
-                self.progressbar.set_text(_("Couldn't start rendering"))
-
-    def _settingsButtonClickedCb(self, unused_button):
-        dialog = ExportSettingsDialog(self.settings)
-        res = dialog.run()
-        dialog.hide()
-        if res == gtk.RESPONSE_ACCEPT:
-            self.settings = dialog.getSettings()
-            self._displaySettings()
-        dialog.destroy()
-
-    def do_destroy(self):
-        gst.debug("cleaning up...")
-        self.bus.remove_signal_watch()
-        gobject.source_remove(self.eosid)
-
-    def _eosCb(self, unused_bus, unused_message):
-        self.rendering = False
-        if self.positionhandler:
-            instance.PiTiVi.playground.disconnect(self.positionhandler)
-            self.positionhandler = 0
-        self.progressbar.set_text(_("Rendering Complete"))
-        self.progressbar.set_fraction(1.0)
-        self.cancelbutton.set_label("gtk-close")
-        
-    def _cancelButtonClickedCb(self, unused_button):
-        self.bin.stopRecording()
-        if self.positionhandler:
-            instance.PiTiVi.playground.disconnect(self.positionhandler)
-            self.positionhandler = 0
-        instance.PiTiVi.playground.pause()
-        self.destroy()
 
